@@ -11,7 +11,11 @@
 
 z.expectation <- function (model, X, Y = NULL) {
 
-  W <- getW(model)
+  # (C) 2008-2011 Leo Lahti and Olli-Pekka Huovilainen          
+  # All rights reserved. 
+  # FreeBSD License (keep this notice)     
+
+    W <- getW(model)
   phi <- getPhi(model)
 
   # Center data
@@ -21,15 +25,14 @@ z.expectation <- function (model, X, Y = NULL) {
 
     Y <- t(centerData(t(Y), rm.na = TRUE))
 
-    # FIXME: use the set.M functions here?
-    S <- solve(t(W$X)%*%solve(phi$X)%*%W$X + t(W$Y)%*%solve(phi$Y)%*%W$Y + diag(ncol(W$X)))
+    phi.inv <- list(X = solve(phi$X), Y = solve(phi$Y))
+    S <- set.M.full2(W, phi.inv)
+    #S <- solve(t(W$X)%*%solve(phi$X)%*%W$X + t(W$Y)%*%solve(phi$Y)%*%W$Y + diag(ncol(W$X)))
 
-    return(S%*%(t(W$X)%*%solve(phi$X)%*%X + t(W$Y)%*%solve(phi$Y)%*%Y))
-  }
-  else {
-    W <- W$total
-    phi <- phi$total
-    S <- solve(t(W) %*% solve(phi) %*% W + diag(ncol(W)))
-    return(S %*% (t(W) %*% solve(phi) %*% X))
+    return( S%*%(t(W$X)%*%phi.inv$X%*%X + t(W$Y)%*%phi.inv$Y%*%Y) )
+  } else {
+    phi.inv <- solve(phi$total)
+    S <- set.M.full(W$total, phi.inv)
+    return(S %*% (t(W$total) %*% phi.inv %*% X))
   }
 }
\ No newline at end of file
